;  Note: must add additional time digit to run over 99 hrs.

;	LIST	p=16F627a		;tell assembler what chip we are using
;	include "P16F627a.inc"		;include the defaults for the chip
;	ERRORLEVEL	0,	-302	;suppress bank selection messages
	;__config 0x3D18			;sets the configuration settings (oscillator type etc.)
 ;  	__config (_CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_CLKOUT & _LVP_OFF & _BODEN_OFF & _MCLRE_OFF)
	LIST	p=16F88		;tell assembler what chip we are using
	include "P16F88.inc"		;include the defaults for the chip
	ERRORLEVEL	0,	-302	;suppress bank selection messages


;==============================================================================
; MPLAB stuff here

	LIST b=5, n=97, t=ON, st=OFF
	; absolute listing tabs=5, lines=97, trim long lines=ON, symbol table=OFF

		__CONFIG    _CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_IO

;Program Configuration Register 2
		__CONFIG    _CONFIG2, _IESO_OFF & _FCMEN_OFF


   


;  Define Information
     #DEFINE RS PORTB, 4
     #DEFINE E  PORTB, 7
     #DEFINE TOGGLESW PORTA, 1
;***********************************************************************
;                    FRROM WORKTIMER
LCD_PORT	Equ	PORTB
LCD_TRIS	Equ	TRISB
LCD_RS		Equ	0x04			;LCD handshake lines
LCD_RW		Equ	0x06
LCD_E		Equ	0x07
;**************************************************************************


 CBLOCK     0X26
                sec             ; seconds digit
                sec10           ; 10's of second digit
                mins            ; minutes digit
                min10           ; 10's of minutes digit
                hr              ; hours digit
                hr10            ; 10's of hours digit
                w_temp          ; holds W during interrupt
                status_temp     ; holds STATUS during interrupt
                fsr_temp        ; holds FSR during interrupt
                Dlay            ; 8 Bit Delay Variable
               	templcd			;temp store for 4 bit mode
				templcd2
				count			;used in looping routines
				count1			;used in delay routine
				counta			;used in delay routine
				countb	
				oldtime
               ENDC

            ORG 0               ; start at location 0

            goto setup          ; jump over to main routine


		org 0x04
		goto isr
		
		
setup					

	bsf		STATUS,RP0
	 movlw   B'0100000' ;250khz
    bsf     STATUS, RP0 
    movwf   OSCCON 

lp: 
   btfss   OSCCON, 2    
    goto    lp 
    bcf     STATUS, RP0 
	banksel 0				
	clrf PORTB		
	movlw b'00000000'		
	banksel TRISB		
	movwf TRISB		
	banksel 0			
	clrf PORTA			
	movlw b'00000010'	;Will eventually put a switch here
	banksel TRISA		
	movwf TRISA			
	banksel 0			
	bsf     STATUS, RP0					; interrupt setup
	movlw b'11000000'		; GIE=on TOIE=on (timer0 overflow int)
	movwf INTCON			; set up.
	bcf		STATUS, RP0
	movlw	0
	movwf	sec
	movlw	0
	movwf	sec10
	movlw	0
	movwf  mins
	movlw 	0
	movwf	min10
	movlw	0
	movwf	hr
	movlw	0
	movwf	hr10
	
	movlw	b'00001011' ;**************************
	clrf	TMR1H		;*
	movwf	TMR1H		;*	Preloading TMR1H and TMR1L for 1 sec int
	movlw 	b'11101010'	;*	
	clrf	TMR1L		;*
	movwf	TMR1L		;***************************


; enable TIMER1 
	banksel	PIR1
	bcf	PIR1, TMR1IF	; clear timer1 interrupt bit first
	banksel		T1CON
	bsf	T1CON, TMR1ON	; turn on the timer

	banksel		PIE1
	bsf	PIE1, TMR1IE	; turn on the interrupt

	banksel	0
	call LCD_Init
	goto main			

LCD_Init	call	Delay100		;wait for LCD to settle

		movlw	0x20			;Set 4 bit mode
		call	LCD_Cmd

		movlw	0x28			;Set display shift
		call	LCD_Cmd

		movlw	0x06			;Set display character mode
		call	LCD_Cmd

		movlw	0x0c			;Set display on/off and cursor command
		call	LCD_Cmd			;Set cursor off

		call	LCD_Clr			;clear display

		retlw	0x00

LCD_Cmd		movwf	templcd
		swapf	templcd,	w	;send upper nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bcf	LCD_PORT, LCD_RS	;RS line to 1
		call	Pulse_e			;Pulse the E line high

		movf	templcd,	w	;send lower nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bcf	LCD_PORT, LCD_RS	;RS line to 1
		call	Pulse_e			;Pulse the E line high
		call 	Delay5
		retlw	0x00

LCD_CharD	addlw	0x30			;add 0x30 to convert to ASCII
LCD_Char	movwf	templcd
		swapf	templcd,	w	;send upper nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bsf	LCD_PORT, LCD_RS	;RS line to 1
		call	Pulse_e			;Pulse the E line high

		movf	templcd,	w	;send lower nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bsf	LCD_PORT, LCD_RS	;RS line to 1
		call	Pulse_e			;Pulse the E line high
		call 	Delay5
		retlw	0x00

LCD_Line1	movlw	0x80			;move to 1st row, first column
		call	LCD_Cmd
		retlw	0x00

LCD_Line2	movlw	0xc0			;move to 2nd row, first column
		call	LCD_Cmd
		retlw	0x00

LCD_Line1W	addlw	0x80			;move to 1st row, column W
		call	LCD_Cmd
		retlw	0x00

LCD_Line2W	addlw	0xc0			;move to 2nd row, column W
		call	LCD_Cmd
		retlw	0x00

LCD_CurOn	movlw	0x0d			;Set display on/off and cursor command
		call	LCD_Cmd
		retlw	0x00

LCD_CurOff	movlw	0x0c			;Set display on/off and cursor command
		call	LCD_Cmd
		retlw	0x00

LCD_Clr		movlw	0x01			;Clear display
		call	LCD_Cmd
		retlw	0x00



Delay255	movlw	0xff		;delay 255 mS
		goto	d0
Delay100	movlw	d'100'		;delay 100mS
		goto	d0
Delay50		movlw	d'50'		;delay 50mS
		goto	d0
Delay20		movlw	d'20'		;delay 20mS
		goto	d0
Delay5		movlw	0x05		;delay 5.000 ms (4 MHz clock)
d0		movwf	count1
d1		movlw	0xC7			;delay 1mS
		movwf	counta
		movlw	0x01
		movwf	countb
Delay_0
		decfsz	counta, f
		goto	$+2
		decfsz	countb, f
		goto	Delay_0

		decfsz	count1	,f
		goto	d1
		retlw	0x00

Pulse_e		bsf	LCD_PORT, LCD_E
		nop
		bcf	LCD_PORT, LCD_E
		retlw	0x00


;-------------------------------------------------------------------------;
;                    Data for message to be output                        ;
;-------------------------------------------------------------------------;
shomsg                          ;  Message to Output
  addwf  PCL, f                 ;  Output the Characters
  dt     "Working Time:", 0

;-------------------------------------------------------------------------;
;                      High limit + 1 of digits at position W             ;
;-------------------------------------------------------------------------;
sethi:
            addwf PCL, f
         dt H'A',H'6',H'A',H'6',H'A',H'A'
         
 ;*********************************************************        
         
OutMessage:
  movwf  FSR                    ;  Point at first letter
OutLoop:
  movf   FSR, w                 ;  Get pointer into W
  incf   FSR, f                 ;  Set up for next letter
  call   shomsg                 ;  Get character to output
  iorlw  0                      ;  At the End of the Message?
  btfsc  STATUS, Z              ;  Skip if not at end
  return                        ;  Yes - Equal to Zero
  call   LCD_Char              ;  Output the ASCII Character
  goto   OutLoop                ;  Get the next character
;***************************************************************************
;               Interrupt Routine
isr:
            movwf w_temp             ; save W
            swapf STATUS,W           ; save status
            movwf status_temp        ; without changing flags
            swapf FSR,W              ; save FSR
            movwf fsr_temp           ; without changing flags
;Disable TIMER1			
			banksel	T1CON	
			bcf		T1CON,TMR1ON	
			banksel	0
  			 movlw	b'00001011'
			clrf	TMR1H
			movwf	TMR1H			;Preloading TMR1H & L for 1 sec int
			movlw 	b'11101111'
			clrf	TMR1L
			movwf	TMR1L
		


;Enable TIMER1 
			banksel	PIR1
			bcf	PIR1, TMR1IF	; clear timer1 interrupt bit 
			banksel		T1CON
			bsf	T1CON, TMR1ON	; turn on TMR1
		
			banksel		PIE1
			bsf	PIE1, TMR1IE	; turn on the interrupt

			banksel	0
            movlw sec                ; point at sec register
            movwf FSR
newdigit:   incf INDF, f             ; current digit up one
            movlw sec                ; get difference between sec and FSR
            subwf FSR, W
            call sethi               ; use to get high limit + 1
            subwf INDF, W            ; reached that number yet?
            btfss STATUS, Z          ; skip over if yes
            goto restore             ; else exit isr
            clrf INDF                ; set current digit to 0
            incf FSR, f               ; point at next digit
            goto newdigit              ; no, increment the next digit
		;	call	DispTime

		
restore:
            swapf status_temp,W      ; get original status back
            movwf STATUS             ; into status register
            swapf fsr_temp,W         ; get original fsr back
            movwf FSR                ; into status register
            swapf w_temp,f           ; old no flags trick again
            swapf w_temp,W           ; to restore W
          
            retfie                   ; finished reset GIE


;-------------------------------------------------------------------------;
;                       Display the Time                                  ;
;-------------------------------------------------------------------------;
DispTime
              MOVLW 3
              CALL LCD_Line2W
              MOVF hr10, W
              CALL LCD_CharD
              MOVF hr, W
              CALL LCD_CharD
              MOVLW ":"
              CALL LCD_Char
              MOVF min10, W
              CALL LCD_CharD
              MOVF mins, W
              CALL LCD_CharD
              MOVLW ":"
              CALL LCD_Char
              MOVF sec10, W
              CALL LCD_CharD
              MOVF sec, W
              CALL LCD_CharD
              RETURN

;-------------------------------------------------------------------------;
nmsec:                         ; delay for # msec in W on entry
         nop                   ; each nop is 0.122 milliseconds
         nop
         nop                   ; each total loop is 8 X 0.122 = 0.976 msec
         nop
         addlw H'FF'           ; same as subtracting 1 from W
         btfss STATUS, Z       ; skip if result is zero
         goto nmsec            ; this is  2 X 0.122 msec
         return                ; back to calling point
;******************************************************************




main
	
	call OutMessage
	call DispTime
	 
loop                      
  
       movf oldtime, W         ; is oldtime the same as sec?
       subwf sec, W
       btfsc STATUS, Z         ; if not, skip over next instruction
       goto loop           ; else continue checking
 
       movf sec, W             ; make sec and oldsec the same
       movwf oldtime
     	call	DispTime
       goto loop       ; and continue checking

	

	end
